@@ -1,12 +1,13 @@
-from typing import Any, Generic, Mapping, Optional
+from typing import Any, Callable, Generic, Mapping, Optional
 from ..analysis.base import AbstractStatistics, StatsMixin, AnnotationStatisticsSlim
 from ..activelearning.base import ActiveLearner
 from ..typehints.typevars import KT, LT
 from .base import AbstractStopCriterion
-from scipy.stats import hypergeom
+from scipy.stats import hypergeom  # type: ignore
 import numpy as np
 import numpy.typing as npt
 import instancelib as il
+from typing_extensions import Self
 
 
 class StatsStoppingCriterion(AbstractStopCriterion[LT], Generic[KT, LT]):
@@ -34,6 +35,13 @@ class StopAfterKNegative(StatsStoppingCriterion[KT, LT]):
         annotated_since_last_pos = self.stats.annotations_since_last(self.pos_label)
         return annotated_since_last_pos >= self.k
 
+    @classmethod
+    def builder(cls, k: int) -> Callable[[LT, LT], Self]:
+        def func(pos_label: LT, neg_label: LT) -> Self:
+            return cls(pos_label, k)
+
+        return func
+
 
 class KneeStoppingRule(StatsStoppingCriterion[KT, LT]):
     """
@@ -43,25 +51,31 @@ class KneeStoppingRule(StatsStoppingCriterion[KT, LT]):
                `<https://dl.acm.org/doi/10.1145/2911451.2911510>`__
     """
 
+    @property
+    def rho(self) -> float:
+        pos_per_round = np.array(self.stats.label_per_round(self.pos_label))
+        Lp = pos_per_round.cumsum()
+        annotated_per_round = np.array(self.stats.annotations_per_round)
+        L = annotated_per_round.cumsum()
+        rho_s = -1
+        for i in range(self.stats.rounds):
+            rho = (Lp[i] / L[-1]) * ((L[-1] - L[i]) / (1 + Lp[-1] - Lp[i]))
+            rho_s = max(rho_s, rho)
+        return rho_s
+
     @property
     def stop_criterion(self) -> bool:
         if self.stats.rounds < 1:
             return False
+        if self.stats.current_annotated < 1000:
+            return False
 
         pos_per_round = np.array(self.stats.label_per_round(self.pos_label))
-        pos_found = pos_per_round.cumsum()
-
-        rho_s = -1
-        for i in range(self.stats.rounds):
-            rho = (pos_found[i] / (i + 1)) / (
-                (1 + pos_found[-1] - pos_found[i]) / (self.stats.rounds - i)
-            )
-            rho_s = max(rho_s, rho)
-
-        return rho_s >= 156 - min(pos_found[-1], 150)
+        Lp = pos_per_round.cumsum()
+        return self.rho >= (156 - min(Lp[-1], 150))
 
 
-class BudgetStoppingRule(StatsStoppingCriterion[KT, LT]):
+class BudgetStoppingRule(KneeStoppingRule[KT, LT], Generic[KT, LT]):
     """
     .. seealso::
         .. [2] Gordon V. Cormack, and Maura R. Grossman. "Engineering quality and reliability in technology-assisted review."
@@ -69,34 +83,30 @@ class BudgetStoppingRule(StatsStoppingCriterion[KT, LT]):
                `<https://dl.acm.org/doi/10.1145/2911451.2911510>`__
     """
 
+    def __init__(self, pos_label: LT, target_size: int = 10) -> None:
+        super().__init__(pos_label)
+        self.target_size = target_size
+
     @property
     def stop_criterion(self) -> bool:
-        if self.stats.rounds < 1:
+        if self.stats.rounds < 1 or self.stats.current_label_count(self.pos_label) < 1:
             return False
 
-        batchsize = self.stats.annotations_per_round[-1]
-        pos_per_round = np.array([self.stats.label_per_round(self.pos_label)])
-        pos_found = pos_per_round.cumsum()
+        pos_per_round = np.array(self.stats.label_per_round(self.pos_label))
+        Lp = pos_per_round.cumsum()
         n_docs = self.stats.dataset_size
-        n_rounds = self.stats.rounds
-        rho_s = -1
-        for i in range(n_rounds):
-            rho = (pos_found[i] / (i + 1)) / (
-                (1 + pos_found[-1] - pos_found[i]) / (n_rounds - i)
-            )
-            rho_s = max(rho_s, rho)
-
-        return (
-            rho_s >= 6
-            and batchsize * self.stats.rounds + 1
-            >= 10 * n_docs / pos_found[n_rounds - 1]
-        ) or (self.stats.current_annotated >= n_docs * 0.75)
+        condition_one = self.stats.current_annotated >= n_docs * 0.75
+        condition_two_a = self.rho >= 6
+        condition_two_b = self.stats.current_annotated >= self.target_size * (
+            n_docs / Lp[-1]
+        )
+        return condition_one or (condition_two_a and condition_two_b)
 
 
 class ReviewHalfStoppingRule(StatsStoppingCriterion[KT, LT]):
     @property
     def stop_criterion(self) -> bool:
-        if self.stats.rounds < 1:
+        if self.stats.rounds < 1 or self.stats.current_label_count(self.pos_label) < 1:
             return False
         return self.stats.current_annotated >= self.stats.dataset_size // 2
 
@@ -164,7 +174,7 @@ class QuantStoppingRule(StatsStoppingCriterion[KT, LT]):
             self.unknown_ps = sum([self.scores[k] for k in learner.env.unlabeled])
             self.known_ps = sum([self.scores[k] for k in learner.env.labeled])
             self.unknown_var = sum(
-                [self.scores[k] * (1 - self.scores[k]) for k in learner.env.labeled]
+                [self.scores[k] * (1 - self.scores[k]) for k in learner.env.unlabeled]
             )
             self.all_var = sum([s * (1 - s) for s in self.scores.values()])
 
@@ -190,7 +200,7 @@ class QuantStoppingRule(StatsStoppingCriterion[KT, LT]):
         return est_recall - self.nstd * np.sqrt(est_var) >= self.target_recall
 
 
-class CHMHeuristicsStoppingRule(StatsStoppingCriterion[KT, LT]):
+class CMH_HeuristicStoppingRule(StatsStoppingCriterion[KT, LT]):
     """
     .. seealso::
         .. [4] Max W. Callaghan, and Finn Müller-Hansen. "Statistical stopping criteria for automated screening in systematic reviews."
